/*****************************************
 * name : av_hwdec.c
 * function: decoder hardware relative functions
 *date      : 2010.2.4
 *****************************************/

#include "gst/gst.h"
#include "gstamlvout.h"
#include "../include/codec.h"
#include  "gstamlvideoheader.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <unistd.h> 
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <errno.h>
#include <stdbool.h>
#include <ctype.h>

static int check_size_in_buffer(unsigned char *p, int len)
{
    unsigned int size;
    unsigned char *q = p;
    while ((q + 4) < (p + len)) {
        size = (*q << 24) | (*(q + 1) << 16) | (*(q + 2) << 8) | (*(q + 3));
        if (size & 0xff000000) {
            return 0;
        }

        if (q + size + 4 == p + len) {
            return 1;
        }

        q += size + 4;
    }
    return 0;
}

static int check_size_in_buffer3(unsigned char *p, int len)
{
    unsigned int size;
    unsigned char *q = p;
    while ((q + 3) < (p + len)) {
        size = (*q << 16) | (*(q + 1) << 8) | (*(q + 2));

        if (q + size + 3 == p + len) {
            return 1;
        }

        q += size + 3;
    }
    return 0;
}

static int check_size_in_buffer2(unsigned char *p, int len)
{
    unsigned int size;
    unsigned char *q = p;
    while ((q + 2) < (p + len)) {
        size = (*q << 8) | (*(q + 1));

        if (q + size + 2 == p + len) {
            return 1;
        }

        q += size + 2;
    }
    return 0;
}

static const guint32 sample_rates[] = {
    96000, 88200, 64000, 48000, 44100, 32000,
    24000, 22050, 16000, 12000, 11025, 8000
};

/*************************************************************************/
static int mjpeg_write_header(codec_para_t *vpcodec)
{
    const unsigned char mjpeg_addon_data[] = {
        0xff, 0xd8, 0xff, 0xc4, 0x01, 0xa2, 0x00, 0x00, 0x01, 0x05, 0x01, 0x01, 0x01,
        0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02,
        0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x01, 0x00, 0x03, 0x01,
        0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x10,
        0x00, 0x02, 0x01, 0x03, 0x03, 0x02, 0x04, 0x03, 0x05, 0x05, 0x04, 0x04, 0x00,
        0x00, 0x01, 0x7d, 0x01, 0x02, 0x03, 0x00, 0x04, 0x11, 0x05, 0x12, 0x21, 0x31,
        0x41, 0x06, 0x13, 0x51, 0x61, 0x07, 0x22, 0x71, 0x14, 0x32, 0x81, 0x91, 0xa1,
        0x08, 0x23, 0x42, 0xb1, 0xc1, 0x15, 0x52, 0xd1, 0xf0, 0x24, 0x33, 0x62, 0x72,
        0x82, 0x09, 0x0a, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x25, 0x26, 0x27, 0x28, 0x29,
        0x2a, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x43, 0x44, 0x45, 0x46, 0x47,
        0x48, 0x49, 0x4a, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5a, 0x63, 0x64,
        0x65, 0x66, 0x67, 0x68, 0x69, 0x6a, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79,
        0x7a, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89, 0x8a, 0x92, 0x93, 0x94, 0x95,
        0x96, 0x97, 0x98, 0x99, 0x9a, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8, 0xa9,
        0xaa, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7, 0xb8, 0xb9, 0xba, 0xc2, 0xc3, 0xc4,
        0xc5, 0xc6, 0xc7, 0xc8, 0xc9, 0xca, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7, 0xd8,
        0xd9, 0xda, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea, 0xf1,
        0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0xfa, 0x11, 0x00, 0x02, 0x01,
        0x02, 0x04, 0x04, 0x03, 0x04, 0x07, 0x05, 0x04, 0x04, 0x00, 0x01, 0x02, 0x77,
        0x00, 0x01, 0x02, 0x03, 0x11, 0x04, 0x05, 0x21, 0x31, 0x06, 0x12, 0x41, 0x51,
        0x07, 0x61, 0x71, 0x13, 0x22, 0x32, 0x81, 0x08, 0x14, 0x42, 0x91, 0xa1, 0xb1,
        0xc1, 0x09, 0x23, 0x33, 0x52, 0xf0, 0x15, 0x62, 0x72, 0xd1, 0x0a, 0x16, 0x24,
        0x34, 0xe1, 0x25, 0xf1, 0x17, 0x18, 0x19, 0x1a, 0x26, 0x27, 0x28, 0x29, 0x2a,
        0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49,
        0x4a, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5a, 0x63, 0x64, 0x65, 0x66,
        0x67, 0x68, 0x69, 0x6a, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7a, 0x82,
        0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89, 0x8a, 0x92, 0x93, 0x94, 0x95, 0x96,
        0x97, 0x98, 0x99, 0x9a, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8, 0xa9, 0xaa,
        0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7, 0xb8, 0xb9, 0xba, 0xc2, 0xc3, 0xc4, 0xc5,
        0xc6, 0xc7, 0xc8, 0xc9, 0xca, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7, 0xd8, 0xd9,
        0xda, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea, 0xf2, 0xf3, 0xf4,
        0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0xfa
    };
    codec_write(vpcodec,&mjpeg_addon_data,sizeof(mjpeg_addon_data));
    return 0;
}
/*************************************************************************/
static int divx3_write_header(codec_para_t *vpcodec)
{
    unsigned i = (vpcodec->am_sysinfo.width<< 12) | (vpcodec->am_sysinfo.height & 0xfff);
    unsigned char divx311_add[10] = {
        0x00, 0x00, 0x00, 0x01,
        0x20, 0x00, 0x00, 0x00,
        0x00, 0x00
    };
    divx311_add[5] = (i >> 16) & 0xff;
    divx311_add[6] = (i >> 8) & 0xff;
    divx311_add[7] = i & 0xff;
    codec_write(vpcodec,&divx311_add,sizeof(divx311_add));
    return 0;
}
/*************************************************************************/
static int h264_add_header(GstAmlVout *Gstamlvout,GstBuffer **buffer)
{
    char nal_start_code[] = {0x0, 0x0, 0x0, 0x1};
    int nalsize;
    unsigned char* p;
    int tmpi;
    guint size = GST_BUFFER_SIZE(Gstamlvout->codec_data);	
    unsigned char* extradata = GST_BUFFER_DATA(Gstamlvout->codec_data);
    int header_len = 0;
    //char* buffer = pkt->hdr->data;
    GstBuffer *hdrBuf=NULL;
    unsigned char *bufout = NULL;  
    g_print("add 264 header in stream\n");
    p = extradata;
    if ((p[0] == 0 && p[1] == 0 && p[2] == 0 && p[3] == 1)||(p[0] == 0 && p[1] == 0 && p[2] == 1 ) && size < 1024) {
        g_print("add 264 header in stream before header len=%d", size);
         hdrBuf = gst_buffer_copy(Gstamlvout->codec_data);
	  *buffer = gst_buffer_merge(hdrBuf,*buffer);	 
        return 0;
    }
    if (size < 4) {
        return -1;
    }

    if (size < 10) {
        g_print("avcC too short\n");
        return -1;
    }

    if (*p != 1) {
        g_print(" Unkonwn avcC version %d\n", *p);
        return -1;
    }
    hdrBuf = gst_buffer_new_and_alloc(Gstamlvout->codec_data_len);
    bufout = GST_BUFFER_DATA(hdrBuf);
    int cnt = *(p + 5) & 0x1f; //number of sps
    // printf("number of sps :%d\n", cnt);
    p += 6;
    for (tmpi = 0; tmpi < cnt; tmpi++) {
        nalsize = (*p << 8) | (*(p + 1));
        memcpy(&(bufout[header_len]), nal_start_code, 4);
        header_len += 4;
        memcpy(&(bufout[header_len]), p + 2, nalsize);
        header_len += nalsize;
        p += (nalsize + 2);
    }

    cnt = *(p++); //Number of pps
    // printf("number of pps :%d\n", cnt);
    for (tmpi = 0; tmpi < cnt; tmpi++) {
        nalsize = (*p << 8) | (*(p + 1));
        memcpy(&(bufout[header_len]), nal_start_code, 4);
        header_len += 4;
        memcpy(&(bufout[header_len]), p + 2, nalsize);
        header_len += nalsize;
        p += (nalsize + 2);
    }
    if (header_len >= 1024) {
        g_print("header_len %d is larger than max length\n", header_len);
        return -1;
    }
    GST_BUFFER_SIZE(hdrBuf) = header_len;
    h264_update_frame_header(buffer);
    *buffer = gst_buffer_merge(hdrBuf,*buffer);
    if(hdrBuf)
        gst_buffer_unref(hdrBuf);
    return 0;
}

static int h264_write_header(GstAmlVout *Gstamlvout,GstBuffer **buffer)
{    
    h264_add_header(Gstamlvout, buffer);  
    return 0;
}

/*************************************************************************/
static int m4s2_dx50_mp4v_write_header(GstAmlVout *Gstamlvout,codec_para_t *vpcodec)
{
    codec_write(vpcodec,GST_BUFFER_DATA(Gstamlvout->codec_data),GST_BUFFER_SIZE(Gstamlvout->codec_data));
    return 0;
}
/*************************************************************************/
static int wmv3_write_header(GstAmlVout *Gstamlvout,codec_para_t *vpcodec)
{
    unsigned i, check_sum = 0;
    guint32 data_len = GST_BUFFER_SIZE(Gstamlvout->codec_data) + 4;
    GstBuffer *hdrBuf=NULL;
    unsigned char *bufout = NULL;  
    

    hdrBuf = gst_buffer_new_and_alloc(26);
    bufout = GST_BUFFER_DATA(hdrBuf);

    bufout[0] = 0;
    bufout[1] = 0;
    bufout[2] = 1;
    bufout[3] = 0x10;

    bufout[4] = 0;
    bufout[5] = (data_len >> 16) & 0xff;
    bufout[6] = 0x88;
    bufout[7] = (data_len >> 8) & 0xff;
    bufout[8] = data_len & 0xff;
    bufout[9] = 0x88;

    bufout[10] = 0xff;
    bufout[11] = 0xff;
    bufout[12] = 0x88;
    bufout[13] = 0xff;
    bufout[14] = 0xff;
    bufout[15] = 0x88;

    for (i = 4 ; i < 16 ; i++) {
        check_sum += bufout[i];
    }

    bufout[16] = (check_sum >> 8) & 0xff;
    bufout[17] = check_sum & 0xff;
    bufout[18] = 0x88;
    bufout[19] = (check_sum >> 8) & 0xff;
    bufout[20] = check_sum & 0xff;
    bufout[21] = 0x88;

    bufout[22] = (vpcodec->am_sysinfo.width >> 8) & 0xff;
    bufout[23] = vpcodec->am_sysinfo.width & 0xff;
    bufout[24] = (vpcodec->am_sysinfo.height >> 8) & 0xff;
    bufout[25] = vpcodec->am_sysinfo.height & 0xff;

    hdrBuf = gst_buffer_merge(hdrBuf,Gstamlvout->codec_data);	 
    GST_BUFFER_SIZE(hdrBuf) = data_len + 26-4;

    codec_write(vpcodec,GST_BUFFER_DATA(hdrBuf),GST_BUFFER_SIZE(hdrBuf));
    if(hdrBuf)	
        gst_buffer_unref(hdrBuf);
    return 0;
}
/*************************************************************************/
static int wvc1_write_header(GstAmlVout *Gstamlvout,codec_para_t*vpcodec)
{    
    codec_write(vpcodec,GST_BUFFER_DATA(Gstamlvout->codec_data)+1,GST_BUFFER_SIZE(Gstamlvout->codec_data)-1);
    return 0;
}
/*************************************************************************/
 GstFlowReturn videopre_header_feeding(codec_para_t *vpcodec,GstAmlVout *Gstamlvout,GstBuffer ** buf)
{ 
    gint ret;
    if (vpcodec&&buf) {
        g_print("-----preheader feeding\n");		
        if (VFORMAT_MJPEG == vpcodec->video_type) {
            ret = mjpeg_write_header(vpcodec);
            if (ret != 0) {
                return ret;
            }
        } else if ((VFORMAT_MPEG4 == vpcodec->video_type) &&
                   (VIDEO_DEC_FORMAT_MPEG4_3 == vpcodec->am_sysinfo.format)) {
            ret = divx3_write_header(vpcodec);
            if (ret != 0) {
                return ret;
            }
        } else if ((VFORMAT_H264 == vpcodec->video_type) || (VFORMAT_H264MVC == vpcodec->video_type)) {
            ret = h264_write_header(Gstamlvout, buf);
            if (ret != 0) {
                return ret;
            }
        } else if ((VFORMAT_MPEG4 == vpcodec->video_type) &&
                   (VIDEO_DEC_FORMAT_MPEG4_5 == vpcodec->am_sysinfo.format)) {
            ret = m4s2_dx50_mp4v_write_header(Gstamlvout, vpcodec);
            if (ret !=  0) {
                return ret;
            }
        }
#if 0
		else if (CODEC_TAG_WMV3 == avcodec->codec_tag) {
            ret = wmv3_write_header(Gstamlvout, vpcodec);
            if (ret != 0) {
                return ret;
            }
        } else if ((CODEC_TAG_WVC1 == avcodec->codec_tag) || (CODEC_TAG_WMVA == avcodec->codec_tag)) {
            ret = wvc1_write_header(Gstamlvout,vpcodec);
            if (ret != 0) {
                return ret;
            }
        } 
#endif		
    } 
    return 0;
}

int h264_update_frame_header(GstBuffer * *buffer)
{
    int nalsize, size = GST_BUFFER_SIZE(*buffer);
    unsigned char *data = GST_BUFFER_DATA(*buffer);;
    unsigned char *p = data; 
    GstBuffer *hdrBuf;
    //g_print("h264_update_frame_header\n");
    if (p != NULL) {
        if (check_size_in_buffer(p, size)) {
            while ((p + 4) < (data + size)) {
                nalsize = (*p << 24) | (*(p + 1) << 16) | (*(p + 2) << 8) | (*(p + 3));
                *p = 0;
                *(p + 1) = 0;
                *(p + 2) = 0;
                *(p + 3) = 1;
                p += (nalsize + 4);
            }
            return 0;
        } else if (check_size_in_buffer3(p, size)) {
            while ((p + 3) < (data + size)) {
                nalsize = (*p << 16) | (*(p + 1) << 8) | (*(p + 2));
                *p = 0;
                *(p + 1) = 0;
                *(p + 2) = 1;
                p += (nalsize + 3);
            }
            return 0;
        } else if (check_size_in_buffer2(p, size)) {
            unsigned char *new_data;
            int new_len = 0;
            hdrBuf = gst_buffer_new_and_alloc(size + 2 * 1024);
            //bufout = GST_BUFFER_DATA(hdrBuf);
            new_data = GST_BUFFER_DATA(hdrBuf);;
            if (!new_data) {
                return -1;
            }

            while ((p + 2) < (data + size)) {
                nalsize = (*p << 8) | (*(p + 1));
                *(new_data + new_len) = 0;
                *(new_data + new_len + 1) = 0;
                *(new_data + new_len + 2) = 0;
                *(new_data + new_len + 3) = 1;
                memcpy(new_data + new_len + 4, p + 2, nalsize);
                p += (nalsize + 2);
                new_len += nalsize + 4;
            }
            gst_buffer_set_data(*buffer,new_data,size + 2 * 1024);
            GST_BUFFER_SIZE(*buffer)= new_len;
            if(hdrBuf)	
            gst_buffer_unref(hdrBuf);   
        }
    } else {
        g_print("[%s]invalid pointer!\n", __FUNCTION__);
        return -1;
    }
   return 0; 
}

#if 0
int divx3_prefix(am_packet_t *pkt)
{
#define DIVX311_CHUNK_HEAD_SIZE 13
    const unsigned char divx311_chunk_prefix[DIVX311_CHUNK_HEAD_SIZE] = {
        0x00, 0x00, 0x00, 0x01, 0xb6, 'D', 'I', 'V', 'X', '3', '.', '1', '1'
    };
    if ((pkt->hdr != NULL) && (pkt->hdr->data != NULL)) {
        FREE(pkt->hdr->data);
        pkt->hdr->data = NULL;
    }

    if (pkt->hdr == NULL) {
        pkt->hdr = MALLOC(sizeof(hdr_buf_t));
        if (!pkt->hdr) {
            log_print("[divx3_prefix] NOMEM!");
            return PLAYER_FAILED;
        }

        pkt->hdr->data = NULL;
        pkt->hdr->size = 0;
    }

    pkt->hdr->data = MALLOC(DIVX311_CHUNK_HEAD_SIZE + 4);
    if (pkt->hdr->data == NULL) {
        log_print("[divx3_prefix] NOMEM!");
        return PLAYER_FAILED;
    }

    MEMCPY(pkt->hdr->data, divx311_chunk_prefix, DIVX311_CHUNK_HEAD_SIZE);

    pkt->hdr->data[DIVX311_CHUNK_HEAD_SIZE + 0] = (pkt->data_size >> 24) & 0xff;
    pkt->hdr->data[DIVX311_CHUNK_HEAD_SIZE + 1] = (pkt->data_size >> 16) & 0xff;
    pkt->hdr->data[DIVX311_CHUNK_HEAD_SIZE + 2] = (pkt->data_size >>  8) & 0xff;
    pkt->hdr->data[DIVX311_CHUNK_HEAD_SIZE + 3] = pkt->data_size & 0xff;

    pkt->hdr->size = DIVX311_CHUNK_HEAD_SIZE + 4;
    pkt->avpkt_newflag = 1;

    return PLAYER_SUCCESS;
}

int h264_write_end_header(play_para_t *para)
{
    int ret;
    unsigned char *tmp_data = NULL;
    unsigned char end_header[] = {
        0x00, 0x00, 0x00, 0x01, 0x06, 0x05, 0xff, 0xe4, 0xdc, 0x45, 0xe9, 0xbd, 0xe6, 0xd9, 0x48, 0xb7,
        0x96, 0x2c, 0xd8, 0x20, 0xd9, 0x23, 0xee, 0xef, 0x78, 0x32, 0x36, 0x34, 0x20, 0x2d, 0x20, 0x63,
        0x6f, 0x72, 0x65, 0x20, 0x36, 0x37, 0x20, 0x72, 0x31, 0x31, 0x33, 0x30, 0x20, 0x38, 0x34, 0x37,
        0x35, 0x39, 0x37, 0x37, 0x20, 0x2d, 0x20, 0x48, 0x2e, 0x32, 0x36, 0x34, 0x2f, 0x4d, 0x50, 0x45,
        0x47, 0x2d, 0x34, 0x20, 0x41, 0x56, 0x43, 0x20, 0x63, 0x6f, 0x64, 0x65, 0x63, 0x20, 0x2d, 0x20,
        0x43, 0x6f, 0x70, 0x79, 0x6c, 0x65, 0x66, 0x74, 0x20, 0x32, 0x30, 0x30, 0x33, 0x2d, 0x32, 0x30,
        0x30, 0x39, 0x20, 0x2d, 0x20, 0x68, 0x74, 0x74, 0x70, 0x3a, 0x2f, 0x2f, 0x77, 0x77, 0x77, 0x2e,
        0x76, 0x69, 0x64, 0x65, 0x6f, 0x6c, 0x61, 0x6e, 0x2e, 0x6f, 0x72, 0x67, 0x2f, 0x78, 0x32, 0x36,
        0x34, 0x2e, 0x68, 0x74, 0x6d, 0x6c, 0x20, 0x2d, 0x20, 0x6f, 0x70, 0x74, 0x69, 0x6f, 0x6e, 0x73,
        0x3a, 0x20, 0x63, 0x61, 0x62, 0x61, 0x63, 0x3d, 0x31, 0x20, 0x72, 0x65, 0x66, 0x3d, 0x31, 0x20,
        0x64, 0x65, 0x62, 0x6c, 0x6f, 0x63, 0x6b, 0x3d, 0x31, 0x3a, 0x30, 0x3a, 0x30, 0x20, 0x61, 0x6e,
        0x61, 0x6c, 0x79, 0x73, 0x65, 0x3d, 0x30, 0x78, 0x31, 0x3a, 0x30, 0x78, 0x31, 0x31, 0x31, 0x20,
        0x6d, 0x65, 0x3d, 0x68, 0x65, 0x78, 0x20, 0x73, 0x75, 0x62, 0x6d, 0x65, 0x3d, 0x36, 0x20, 0x70,
        0x73, 0x79, 0x5f, 0x72, 0x64, 0x3d, 0x31, 0x2e, 0x30, 0x3a, 0x30, 0x2e, 0x30, 0x20, 0x6d, 0x69,
        0x78, 0x65, 0x64, 0x5f, 0x72, 0x65, 0x66, 0x3d, 0x30, 0x20, 0x6d, 0x65, 0x5f, 0x72, 0x61, 0x6e,
        0x67, 0x65, 0x3d, 0x31, 0x36, 0x20, 0x63, 0x68, 0x72, 0x6f, 0x6d, 0x61, 0x5f, 0x6d, 0x65, 0x3d,
        0x31, 0x20, 0x74, 0x72, 0x65, 0x6c, 0x6c, 0x69, 0x73, 0x3d, 0x30, 0x20, 0x38, 0x78, 0x38, 0x64,
        0x63, 0x74, 0x3d, 0x30, 0x20, 0x63, 0x71, 0x6d, 0x3d, 0x30, 0x20, 0x64, 0x65, 0x61, 0x64, 0x7a,
        0x6f, 0x6e, 0x65, 0x3d, 0x32, 0x31, 0x2c, 0x31, 0x31, 0x20, 0x63, 0x68, 0x72, 0x6f, 0x6d, 0x61,
        0x5f, 0x71, 0x70, 0x5f, 0x6f, 0x66, 0x66, 0x73, 0x65, 0x74, 0x3d, 0x2d, 0x32, 0x20, 0x74, 0x68,
        0x72, 0x65, 0x61, 0x64, 0x73, 0x3d, 0x31, 0x20, 0x6e, 0x72, 0x3d, 0x30, 0x20, 0x64, 0x65, 0x63,
        0x69, 0x6d, 0x61, 0x74, 0x65, 0x3d, 0x31, 0x20, 0x6d, 0x62, 0x61, 0x66, 0x66, 0x3d, 0x30, 0x20,
        0x62, 0x66, 0x72, 0x61, 0x6d, 0x65, 0x73, 0x3d, 0x30, 0x20, 0x6b, 0x65, 0x79, 0x69, 0x6e, 0x74,
        0x3d, 0x32, 0x35, 0x30, 0x20, 0x6b, 0x65, 0x79, 0x69, 0x6e, 0x74, 0x5f, 0x6d, 0x69, 0x6e, 0x3d,
        0x32, 0x35, 0x20, 0x73, 0x63, 0x65, 0x6e, 0x65, 0x63, 0x75, 0x74, 0x3d, 0x34, 0x30, 0x20, 0x72,
        0x63, 0x3d, 0x61, 0x62, 0x72, 0x20, 0x62, 0x69, 0x74, 0x72, 0x61, 0x74, 0x65, 0x3d, 0x31, 0x30,
        0x20, 0x72, 0x61, 0x74, 0x65, 0x74, 0x6f, 0x6c, 0x3d, 0x31, 0x2e, 0x30, 0x20, 0x71, 0x63, 0x6f,
        0x6d, 0x70, 0x3d, 0x30, 0x2e, 0x36, 0x30, 0x20, 0x71, 0x70, 0x6d, 0x69, 0x6e, 0x3d, 0x31, 0x30,
        0x20, 0x71, 0x70, 0x6d, 0x61, 0x78, 0x3d, 0x35, 0x31, 0x20, 0x71, 0x70, 0x73, 0x74, 0x65, 0x70,
        0x3d, 0x34, 0x20, 0x69, 0x70, 0x5f, 0x72, 0x61, 0x74, 0x69, 0x6f, 0x3d, 0x31, 0x2e, 0x34, 0x30,
        0x20, 0x61, 0x71, 0x3d, 0x31, 0x3a, 0x31, 0x2e, 0x30, 0x30, 0x00, 0x80, 0x00, 0x00, 0x00, 0x01,
        0x67, 0x4d, 0x40, 0x0a, 0x9a, 0x74, 0xf4, 0x20, 0x00, 0x00, 0x03, 0x00, 0x20, 0x00, 0x00, 0x06,
        0x51, 0xe2, 0x44, 0xd4, 0x00, 0x00, 0x00, 0x01, 0x68, 0xee, 0x32, 0xc8, 0x00, 0x00, 0x00, 0x01,
        0x65, 0x88, 0x80, 0x20, 0x00, 0x08, 0x7f, 0xea, 0x6a, 0xe2, 0x99, 0xb6, 0x57, 0xae, 0x49, 0x30,
        0xf5, 0xfe, 0x5e, 0x46, 0x0b, 0x72, 0x44, 0xc4, 0xe1, 0xfc, 0x62, 0xda, 0xf1, 0xfb, 0xa2, 0xdb,
        0xd6, 0xbe, 0x5c, 0xd7, 0x24, 0xa3, 0xf5, 0xb9, 0x2f, 0x57, 0x16, 0x49, 0x75, 0x47, 0x77, 0x09,
        0x5c, 0xa1, 0xb4, 0xc3, 0x4f, 0x60, 0x2b, 0xb0, 0x0c, 0xc8, 0xd6, 0x66, 0xba, 0x9b, 0x82, 0x29,
        0x33, 0x92, 0x26, 0x99, 0x31, 0x1c, 0x7f, 0x9b
    };
    int header_size = sizeof(end_header) / sizeof(unsigned char);

    tmp_data = MALLOC(1024);
    if (!tmp_data) {
        return 0;
    }
    MEMSET(tmp_data, 0, 1024);
    MEMCPY(tmp_data, &end_header, header_size);
    ret = codec_write(para->vcodec, (void *)tmp_data, 1024);
    g_print("[%s:%d]ret %d\n", __FUNCTION__, __LINE__, ret);
    FREE(tmp_data);

    return ret;
}

#endif
